@@ -5,9 +5,9 @@ BEGIN {
       'Module::Compile'     => 'PDL/NiceSlice/ModuleCompile.pm',
    );  # to validate names
 
-   ## $PDL::NiceSlice::engine = $engine_ok{'Filter::Simple'};  # default engine type
+   $PDL::NiceSlice::engine = $engine_ok{'Filter::Simple'};  # default engine type
    ## TODO: Add configuration argument to perldl.conf
-   $PDL::NiceSlice::engine = $engine_ok{'Filter::Util::Call'};  # default engine type
+   ## $PDL::NiceSlice::engine = $engine_ok{'Filter::Util::Call'};  # default engine type
 
    if ( exists $ENV{PDL_NICESLICE_ENGINE} ) {
       my $engine = $ENV{PDL_NICESLICE_ENGINE};
@@ -203,8 +203,14 @@ sub procargs {
   # push @callstack, $txt; # for later error reporting
   my $args = $txt =~ /^\s*$/s ? '' :
     join ',', map {onearg $_} splitprotected ',', $txt;
-  $args =~ s/\s//sg; # get rid of whitespace
+    ## Leave whitespace/newlines in so line count
+    ## is preserved in error messages.  Makes the
+    ## filtered output ugly---iffi the input was
+    ## ugly...
+    ## 
+    ## $args =~ s/\s//sg; # get rid of whitespace
   # pop @callstack; # remove from call stack
+  print STDERR "procargs: returned '($args)'\n" if $PDL::NiceSlice::debug;
   return "($args)";
 }
 
